@@ -9,6 +9,7 @@ from typing import (
     Sequence,
     Tuple,
     TypeVar,
+    Union,
 )
 
 from afancontrol.arduino import (
@@ -17,16 +18,30 @@ from afancontrol.arduino import (
     ArduinoConnection,
     ArduinoName,
     ArduinoPin,
-    ArduinoPWMFan,
 )
+from afancontrol.filters import (
+    MovingMedianFilter,
+    MovingQuantileFilter,
+    NullFilter,
+    TempFilter,
+)
+from afancontrol.logger import logger
 from afancontrol.pwmfan import (
-    BasePWMFan,
+    ArduinoFanPWMRead,
+    ArduinoFanPWMWrite,
+    ArduinoFanSpeed,
+    BaseFanPWMRead,
+    BaseFanPWMWrite,
+    BaseFanSpeed,
     FanInputDevice,
-    LinuxPWMFan,
+    FreeIPMIFanSpeed,
+    LinuxFanPWMRead,
+    LinuxFanPWMWrite,
+    LinuxFanSpeed,
     PWMDevice,
-    PWMFanNorm,
     PWMValue,
 )
+from afancontrol.pwmfannorm import PWMFanNorm, ReadonlyPWMFanNorm
 from afancontrol.temp import CommandTemp, FileTemp, HDDTemp, Temp, TempCelsius
 
 DEFAULT_CONFIG = "/etc/afancontrol/afancontrol.conf"
@@ -46,8 +61,13 @@ DEFAULT_PWM_LINE_END = 240
 
 DEFAULT_NEVER_STOP = True
 
+DEFAULT_WINDOW_SIZE = 3
+
+FilterName = NewType("FilterName", str)
 TempName = NewType("TempName", str)
 FanName = NewType("FanName", str)
+ReadonlyFanName = NewType("ReadonlyFanName", str)
+AnyFanName = Union[FanName, ReadonlyFanName]
 MappingName = NewType("MappingName", str)
 
 T = TypeVar("T")
@@ -129,6 +149,16 @@ DaemonConfig = NamedTuple(
     # fmt: on
 )
 
+FilteredTemp = NamedTuple(
+    "FilteredTemp",
+    # fmt: off
+    [
+        ("temp", Temp),
+        ("filter", TempFilter),
+    ]
+    # fmt: on
+)
+
 ParsedConfig = NamedTuple(
     "ParsedConfig",
     # fmt: off
@@ -136,8 +166,10 @@ ParsedConfig = NamedTuple(
         ("daemon", DaemonConfig),
         ("report_cmd", str),
         ("triggers", TriggerConfig),
+        ("arduino_connections", Mapping[ArduinoName, ArduinoConnection]),
         ("fans", Mapping[FanName, PWMFanNorm]),
-        ("temps", Mapping[TempName, Temp]),
+        ("readonly_fans", Mapping[ReadonlyFanName, ReadonlyPWMFanNorm]),
+        ("temps", Mapping[TempName, FilteredTemp]),
         ("mappings", Mapping[MappingName, FansTempsRelation]),
     ]
     # fmt: on
@@ -154,8 +186,11 @@ def parse_config(config_path: Path, daemon_cli_config: DaemonCLIConfig) -> Parse
     daemon, hddtemp = _parse_daemon(config, daemon_cli_config)
     report_cmd, global_commands = _parse_actions(config)
     arduino_connections = _parse_arduino_connections(config)
-    temps, temp_commands = _parse_temps(config, hddtemp)
+    filters = _parse_filters(config)
+    temps, temp_commands = _parse_temps(config, hddtemp, filters)
     fans = _parse_fans(config, arduino_connections)
+    readonly_fans = _parse_readonly_fans(config, arduino_connections)
+    _check_fans_namespace(fans, readonly_fans)
     mappings = _parse_mappings(config, fans, temps)
 
     return ParsedConfig(
@@ -164,7 +199,9 @@ def parse_config(config_path: Path, daemon_cli_config: DaemonCLIConfig) -> Parse
         triggers=TriggerConfig(
             global_commands=global_commands, temp_commands=temp_commands
         ),
+        arduino_connections=arduino_connections,
         fans=fans,
+        readonly_fans=readonly_fans,
         temps=temps,
         mappings=mappings,
     )
@@ -288,10 +325,63 @@ def _parse_arduino_connections(
     return arduino_connections
 
 
+def _parse_filters(
+    config: configparser.ConfigParser,
+) -> Mapping[FilterName, TempFilter]:
+    filters = {}  # type: Dict[FilterName, TempFilter]
+    for section_name in config.sections():
+        section_name_parts = section_name.split(":", 1)
+
+        if section_name_parts[0].strip().lower() != "filter":
+            continue
+
+        filter_name = FilterName(section_name_parts[1].strip())
+        filter = config[section_name]
+        keys = set(filter.keys())
+
+        filter_type = filter["type"]
+        keys.discard("type")
+
+        if filter_type == "moving_median":
+            window_size = filter.getint("window_size", fallback=DEFAULT_WINDOW_SIZE)
+            keys.discard("window_size")
+
+            f = MovingMedianFilter(window_size=window_size)  # type: TempFilter
+        elif filter_type == "moving_quantile":
+            window_size = filter.getint("window_size", fallback=DEFAULT_WINDOW_SIZE)
+            keys.discard("window_size")
+
+            quantile = filter.getfloat("quantile")
+            keys.discard("quantile")
+            f = MovingQuantileFilter(quantile=quantile, window_size=window_size)
+        else:
+            raise RuntimeError(
+                "Unsupported filter type '%s' for filter '%s'. "
+                "Supported types: `moving_median`, `moving_quantile`."
+                % (filter_type, filter_name)
+            )
+
+        if keys:
+            raise RuntimeError(
+                "Unknown options in the [%s] section: %s" % (section_name, keys)
+            )
+
+        if filter_name in filters:
+            raise RuntimeError(
+                "Duplicate filter section declaration for '%s'" % filter_name
+            )
+        filters[filter_name] = f
+
+    # Empty filters is ok
+    return filters
+
+
 def _parse_temps(
-    config: configparser.ConfigParser, hddtemp: str
-) -> Tuple[Mapping[TempName, Temp], Mapping[TempName, Actions]]:
-    temps = {}  # type: Dict[TempName, Temp]
+    config: configparser.ConfigParser,
+    hddtemp: str,
+    filters: Mapping[FilterName, TempFilter],
+) -> Tuple[Mapping[TempName, FilteredTemp], Mapping[TempName, Actions]]:
+    temps = {}  # type: Dict[TempName, FilteredTemp]
     temp_commands = {}  # type: Dict[TempName, Actions]
     for section_name in config.sections():
         section_name_parts = section_name.split(":", 1)
@@ -359,6 +449,14 @@ def _parse_temps(
                 "Unsupported temp type '%s' for temp '%s'" % (type, temp_name)
             )
 
+        filter_name = temp.get("filter")
+        keys.discard("filter")
+
+        if filter_name is None:
+            filter = NullFilter()
+        else:
+            filter = filters[FilterName(filter_name.strip())].copy()
+
         if keys:
             raise RuntimeError(
                 "Unknown options in the [%s] section: %s" % (section_name, keys)
@@ -368,13 +466,11 @@ def _parse_temps(
             raise RuntimeError(
                 "Duplicate temp section declaration for '%s'" % temp_name
             )
-        temps[temp_name] = t
+        temps[temp_name] = FilteredTemp(temp=t, filter=filter)
         temp_commands[temp_name] = Actions(
             panic=actions_panic, threshold=actions_threshold
         )
 
-    if not temps:
-        raise RuntimeError("No temps found in the config, at least 1 must be specified")
     return temps, temp_commands
 
 
@@ -402,7 +498,9 @@ def _parse_fans(
             keys.discard("pwm")
             keys.discard("fan_input")
 
-            pwmfan = LinuxPWMFan(pwm=pwm, fan_input=fan_input)  # type: BasePWMFan
+            fan_speed = LinuxFanSpeed(fan_input)  # type: BaseFanSpeed
+            pwm_read = LinuxFanPWMRead(pwm)  # type: BaseFanPWMRead
+            pwm_write = LinuxFanPWMWrite(pwm)  # type: BaseFanPWMWrite
         elif fan_type == "arduino":
             arduino_name = ArduinoName(fan["arduino_name"])
             keys.discard("arduino_name")
@@ -414,8 +512,14 @@ def _parse_fans(
             if arduino_name not in arduino_connections:
                 raise ValueError("[arduino:%s] section is missing" % arduino_name)
 
-            pwmfan = ArduinoPWMFan(
-                arduino_connections[arduino_name], pwm_pin=pwm_pin, tacho_pin=tacho_pin
+            fan_speed = ArduinoFanSpeed(
+                arduino_connections[arduino_name], tacho_pin=tacho_pin
+            )
+            pwm_read = ArduinoFanPWMRead(
+                arduino_connections[arduino_name], pwm_pin=pwm_pin
+            )
+            pwm_write = ArduinoFanPWMWrite(
+                arduino_connections[arduino_name], pwm_pin=pwm_pin
             )
         else:
             raise ValueError(
@@ -437,10 +541,10 @@ def _parse_fans(
         keys.discard("pwm_line_end")
 
         for pwm_value in (pwm_line_start, pwm_line_end):
-            if not (pwmfan.min_pwm <= pwm_value <= pwmfan.max_pwm):
+            if not (pwm_read.min_pwm <= pwm_value <= pwm_read.max_pwm):
                 raise RuntimeError(
                     "Incorrect PWM value '%s' for fan '%s': it must be within [%s;%s]"
-                    % (pwm_value, fan_name, pwmfan.min_pwm, pwmfan.max_pwm)
+                    % (pwm_value, fan_name, pwm_read.min_pwm, pwm_read.max_pwm)
                 )
         if pwm_line_start >= pwm_line_end:
             raise RuntimeError(
@@ -456,21 +560,109 @@ def _parse_fans(
         if fan_name in fans:
             raise RuntimeError("Duplicate fan section declaration for '%s'" % fan_name)
         fans[fan_name] = PWMFanNorm(
-            pwmfan,
+            fan_speed,
+            pwm_read,
+            pwm_write,
             pwm_line_start=pwm_line_start,
             pwm_line_end=pwm_line_end,
             never_stop=never_stop,
         )
 
-    if not fans:
-        raise RuntimeError("No fans found in the config, at least 1 must be specified")
     return fans
 
 
+def _parse_readonly_fans(
+    config: configparser.ConfigParser,
+    arduino_connections: Mapping[ArduinoName, ArduinoConnection],
+) -> Mapping[ReadonlyFanName, ReadonlyPWMFanNorm]:
+    readonly_fans = {}  # type: Dict[ReadonlyFanName, ReadonlyPWMFanNorm]
+    for section_name in config.sections():
+        section_name_parts = section_name.split(":", 1)
+
+        if section_name_parts[0].strip().lower() != "readonly_fan":
+            continue
+
+        fan_name = ReadonlyFanName(section_name_parts[1].strip())
+        fan = config[section_name]
+        keys = set(fan.keys())
+
+        fan_type = fan.get("type", fallback=DEFAULT_FAN_TYPE)
+        keys.discard("type")
+
+        if fan_type == "linux":
+            fan_input = FanInputDevice(fan["fan_input"])
+            keys.discard("fan_input")
+            fan_speed = LinuxFanSpeed(fan_input)  # type: BaseFanSpeed
+            pwm_read = None  # type: Optional[BaseFanPWMRead]
+            if "pwm" in fan:
+                pwm = PWMDevice(fan["pwm"])
+                keys.discard("pwm")
+                pwm_read = LinuxFanPWMRead(pwm)
+        elif fan_type == "arduino":
+            arduino_name = ArduinoName(fan["arduino_name"])
+            keys.discard("arduino_name")
+            tacho_pin = ArduinoPin(fan.getint("tacho_pin"))
+            keys.discard("tacho_pin")
+
+            if arduino_name not in arduino_connections:
+                raise ValueError("[arduino:%s] section is missing" % arduino_name)
+
+            fan_speed = ArduinoFanSpeed(
+                arduino_connections[arduino_name], tacho_pin=tacho_pin
+            )
+            pwm_read = None
+            if "pwm_pin" in fan:
+                pwm_pin = ArduinoPin(fan.getint("pwm_pin"))
+                keys.discard("pwm_pin")
+                pwm_read = ArduinoFanPWMRead(
+                    arduino_connections[arduino_name], pwm_pin=pwm_pin
+                )
+        elif fan_type == "freeipmi":
+            name = fan["name"]
+            keys.discard("name")
+
+            ipmi_sensors_extra_args = fan.get("ipmi_sensors_extra_args", fallback="")
+
+            fan_speed = FreeIPMIFanSpeed(
+                name, ipmi_sensors_extra_args=ipmi_sensors_extra_args
+            )
+            pwm_read = None
+        else:
+            raise ValueError(
+                "Unsupported FAN type %s. Supported ones are "
+                "`linux`, `arduino`, `freeipmi`." % fan_type
+            )
+
+        if keys:
+            raise RuntimeError(
+                "Unknown options in the [%s] section: %s" % (section_name, keys)
+            )
+
+        if fan_name in readonly_fans:
+            raise RuntimeError(
+                "Duplicate readonly_fan section declaration for '%s'" % fan_name
+            )
+        readonly_fans[fan_name] = ReadonlyPWMFanNorm(fan_speed, pwm_read)
+
+    return readonly_fans
+
+
+def _check_fans_namespace(
+    fans: Mapping[FanName, PWMFanNorm],
+    readonly_fans: Mapping[ReadonlyFanName, ReadonlyPWMFanNorm],
+) -> None:
+    common_keys = fans.keys() & readonly_fans.keys()
+    if common_keys:
+        raise RuntimeError(
+            "Duplicate fan names has been found between `fan` "
+            "and `readonly_fan` sections: %r" % (list(common_keys),)
+        )
+
+
 def _parse_mappings(
     config: configparser.ConfigParser,
     fans: Mapping[FanName, PWMFanNorm],
-    temps: Mapping[TempName, Temp],
+    temps: Mapping[TempName, FilteredTemp],
 ) -> Mapping[MappingName, FansTempsRelation]:
 
     mappings = {}  # type: Dict[MappingName, FansTempsRelation]
@@ -577,9 +769,9 @@ def _parse_mappings(
             fan_speed_modifier.fan for fan_speed_modifier in relation.fans
         )
     if unused_temps:
-        raise RuntimeError(
-            "The following temps are defined but not used in any mapping: %s"
-            % unused_temps
+        logger.warning(
+            "The following temps are defined but not used in any mapping: %s",
+            unused_temps,
         )
     if unused_fans:
         raise RuntimeError(
