@@ -7,7 +7,6 @@ from afancontrol.arduino import (
     ArduinoConnection,
     ArduinoName,
     ArduinoPin,
-    ArduinoPWMFan,
     pyserial_available,
 )
 from afancontrol.config import (
@@ -18,19 +17,27 @@ from afancontrol.config import (
     FanName,
     FanSpeedModifier,
     FansTempsRelation,
+    FilteredTemp,
     MappingName,
     ParsedConfig,
+    ReadonlyFanName,
     TempName,
     TriggerConfig,
     parse_config,
 )
+from afancontrol.filters import MovingMedianFilter, NullFilter
 from afancontrol.pwmfan import (
+    ArduinoFanPWMRead,
+    ArduinoFanPWMWrite,
+    ArduinoFanSpeed,
     FanInputDevice,
-    LinuxPWMFan,
+    LinuxFanPWMRead,
+    LinuxFanPWMWrite,
+    LinuxFanSpeed,
     PWMDevice,
-    PWMFanNorm,
     PWMValue,
 )
+from afancontrol.pwmfannorm import PWMFanNorm, ReadonlyPWMFanNorm
 from afancontrol.temp import FileTemp, HDDTemp, TempCelsius
 
 
@@ -58,6 +65,7 @@ def test_pkg_conf(pkg_conf: Path):
 
     parsed = parse_config(pkg_conf, daemon_cli_config)
     assert parsed == ParsedConfig(
+        arduino_connections={},
         daemon=DaemonConfig(
             pidfile="/run/afancontrol.pid",
             logfile="/var/log/afancontrol.log",
@@ -82,22 +90,37 @@ def test_pkg_conf(pkg_conf: Path):
         ),
         fans={
             FanName("hdd"): PWMFanNorm(
-                LinuxPWMFan(
-                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2"),
-                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan2_input"),
+                fan_speed=LinuxFanSpeed(
+                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan2_input")
+                ),
+                pwm_read=LinuxFanPWMRead(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2")
+                ),
+                pwm_write=LinuxFanPWMWrite(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2")
                 ),
                 pwm_line_start=PWMValue(100),
                 pwm_line_end=PWMValue(240),
                 never_stop=False,
             )
         },
+        readonly_fans={
+            ReadonlyFanName("cpu"): ReadonlyPWMFanNorm(
+                fan_speed=LinuxFanSpeed(
+                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan1_input")
+                ),
+            ),
+        },
         temps={
-            TempName("mobo"): FileTemp(
-                "/sys/class/hwmon/hwmon0/device/temp1_input",
-                min=TempCelsius(30.0),
-                max=TempCelsius(40.0),
-                panic=None,
-                threshold=None,
+            TempName("mobo"): FilteredTemp(
+                temp=FileTemp(
+                    "/sys/class/hwmon/hwmon0/device/temp1_input",
+                    min=TempCelsius(30.0),
+                    max=TempCelsius(40.0),
+                    panic=None,
+                    threshold=None,
+                ),
+                filter=MovingMedianFilter(window_size=3),
             )
         },
         mappings={
@@ -117,6 +140,11 @@ def test_example_conf(example_conf: Path):
 
     parsed = parse_config(example_conf, daemon_cli_config)
     assert parsed == ParsedConfig(
+        arduino_connections={
+            ArduinoName("mymicro"): ArduinoConnection(
+                ArduinoName("mymicro"), "/dev/ttyACM0", baudrate=115200, status_ttl=5
+            )
+        },
         daemon=DaemonConfig(
             pidfile="/run/afancontrol.pid",
             logfile="/var/log/afancontrol.log",
@@ -145,25 +173,55 @@ def test_example_conf(example_conf: Path):
         ),
         fans={
             FanName("cpu"): PWMFanNorm(
-                LinuxPWMFan(
-                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm1"),
-                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan1_input"),
+                fan_speed=LinuxFanSpeed(
+                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan1_input")
+                ),
+                pwm_read=LinuxFanPWMRead(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm1")
+                ),
+                pwm_write=LinuxFanPWMWrite(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm1")
                 ),
                 pwm_line_start=PWMValue(100),
                 pwm_line_end=PWMValue(240),
                 never_stop=True,
             ),
             FanName("hdd"): PWMFanNorm(
-                LinuxPWMFan(
-                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2"),
-                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan2_input"),
+                fan_speed=LinuxFanSpeed(
+                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan2_input")
+                ),
+                pwm_read=LinuxFanPWMRead(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2")
+                ),
+                pwm_write=LinuxFanPWMWrite(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2")
                 ),
                 pwm_line_start=PWMValue(100),
                 pwm_line_end=PWMValue(240),
                 never_stop=False,
             ),
             FanName("my_arduino_fan"): PWMFanNorm(
-                ArduinoPWMFan(
+                fan_speed=ArduinoFanSpeed(
+                    ArduinoConnection(
+                        ArduinoName("mymicro"),
+                        "/dev/ttyACM0",  # linux
+                        # "/dev/cu.usbmodem14201",  # macos
+                        baudrate=115200,
+                        status_ttl=5,
+                    ),
+                    tacho_pin=ArduinoPin(3),
+                ),
+                pwm_read=ArduinoFanPWMRead(
+                    ArduinoConnection(
+                        ArduinoName("mymicro"),
+                        "/dev/ttyACM0",  # linux
+                        # "/dev/cu.usbmodem14201",  # macos
+                        baudrate=115200,
+                        status_ttl=5,
+                    ),
+                    pwm_pin=ArduinoPin(9),
+                ),
+                pwm_write=ArduinoFanPWMWrite(
                     ArduinoConnection(
                         ArduinoName("mymicro"),
                         "/dev/ttyACM0",  # linux
@@ -172,28 +230,34 @@ def test_example_conf(example_conf: Path):
                         status_ttl=5,
                     ),
                     pwm_pin=ArduinoPin(9),
-                    tacho_pin=ArduinoPin(3),
                 ),
                 pwm_line_start=PWMValue(100),
                 pwm_line_end=PWMValue(240),
                 never_stop=True,
             ),
         },
+        readonly_fans={},
         temps={
-            TempName("hdds"): HDDTemp(
-                "/dev/sd?",
-                min=TempCelsius(35.0),
-                max=TempCelsius(48.0),
-                panic=TempCelsius(55.0),
-                threshold=None,
-                hddtemp_bin="hddtemp",
+            TempName("hdds"): FilteredTemp(
+                temp=HDDTemp(
+                    "/dev/sd?",
+                    min=TempCelsius(35.0),
+                    max=TempCelsius(48.0),
+                    panic=TempCelsius(55.0),
+                    threshold=None,
+                    hddtemp_bin="hddtemp",
+                ),
+                filter=NullFilter(),
             ),
-            TempName("mobo"): FileTemp(
-                "/sys/class/hwmon/hwmon0/device/temp1_input",
-                min=TempCelsius(30.0),
-                max=TempCelsius(40.0),
-                panic=None,
-                threshold=None,
+            TempName("mobo"): FilteredTemp(
+                temp=FileTemp(
+                    "/sys/class/hwmon/hwmon0/device/temp1_input",
+                    min=TempCelsius(30.0),
+                    max=TempCelsius(40.0),
+                    panic=None,
+                    threshold=None,
+                ),
+                filter=NullFilter(),
             ),
         },
         mappings={
@@ -237,6 +301,7 @@ temps = mobo
 """
     parsed = parse_config(path_from_str(config), daemon_cli_config)
     assert parsed == ParsedConfig(
+        arduino_connections={},
         daemon=DaemonConfig(
             pidfile="/run/afancontrol.pid",
             logfile=None,
@@ -261,22 +326,31 @@ temps = mobo
         ),
         fans={
             FanName("case"): PWMFanNorm(
-                LinuxPWMFan(
-                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2"),
-                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan2_input"),
+                fan_speed=LinuxFanSpeed(
+                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan2_input")
+                ),
+                pwm_read=LinuxFanPWMRead(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2")
+                ),
+                pwm_write=LinuxFanPWMWrite(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm2")
                 ),
                 pwm_line_start=PWMValue(100),
                 pwm_line_end=PWMValue(240),
                 never_stop=True,
             )
         },
+        readonly_fans={},
         temps={
-            TempName("mobo"): FileTemp(
-                "/sys/class/hwmon/hwmon0/device/temp1_input",
-                min=None,
-                max=None,
-                panic=None,
-                threshold=None,
+            TempName("mobo"): FilteredTemp(
+                temp=FileTemp(
+                    "/sys/class/hwmon/hwmon0/device/temp1_input",
+                    min=None,
+                    max=None,
+                    panic=None,
+                    threshold=None,
+                ),
+                filter=NullFilter(),
             )
         },
         mappings={
@@ -286,3 +360,73 @@ temps = mobo
             )
         },
     )
+
+
+def test_readonly_config() -> None:
+    daemon_cli_config = DaemonCLIConfig(
+        pidfile=None, logfile=None, exporter_listen_host=None
+    )
+
+    config = """
+[daemon]
+
+[actions]
+
+[temp:mobo]
+type = file
+path = /sys/class/hwmon/hwmon0/device/temp1_input
+
+[readonly_fan: cpu]
+pwm = /sys/class/hwmon/hwmon0/device/pwm1
+fan_input = /sys/class/hwmon/hwmon0/device/fan1_input
+"""
+    parsed = parse_config(path_from_str(config), daemon_cli_config)
+    assert parsed == ParsedConfig(
+        arduino_connections={},
+        daemon=DaemonConfig(
+            pidfile="/run/afancontrol.pid",
+            logfile=None,
+            exporter_listen_host=None,
+            interval=5,
+        ),
+        report_cmd=(
+            'printf "Subject: %s\nTo: %s\n\n%b" '
+            '"afancontrol daemon report: %REASON%" root "%MESSAGE%" | sendmail -t'
+        ),
+        triggers=TriggerConfig(
+            global_commands=Actions(
+                panic=AlertCommands(enter_cmd=None, leave_cmd=None),
+                threshold=AlertCommands(enter_cmd=None, leave_cmd=None),
+            ),
+            temp_commands={
+                TempName("mobo"): Actions(
+                    panic=AlertCommands(enter_cmd=None, leave_cmd=None),
+                    threshold=AlertCommands(enter_cmd=None, leave_cmd=None),
+                )
+            },
+        ),
+        fans={},
+        readonly_fans={
+            ReadonlyFanName("cpu"): ReadonlyPWMFanNorm(
+                fan_speed=LinuxFanSpeed(
+                    FanInputDevice("/sys/class/hwmon/hwmon0/device/fan1_input")
+                ),
+                pwm_read=LinuxFanPWMRead(
+                    PWMDevice("/sys/class/hwmon/hwmon0/device/pwm1")
+                ),
+            )
+        },
+        temps={
+            TempName("mobo"): FilteredTemp(
+                temp=FileTemp(
+                    "/sys/class/hwmon/hwmon0/device/temp1_input",
+                    min=None,
+                    max=None,
+                    panic=None,
+                    threshold=None,
+                ),
+                filter=NullFilter(),
+            )
+        },
+        mappings={},
+    )
